"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const config_1 = require("../config");
const cli_ux_1 = require("cli-ux");
const got = require("got");
const lodash_1 = require("lodash");
const generator_1 = require("../helpers/generator");
// This is a the registry of default gists
const defaultGists = {
    'cloudbuild.yaml': '37bb93af644ec7cecb5819323ba1b1bc',
    'dockerize-theia': '531f9c60daee31274e38404361a8f21a',
    jake: '5d1d7287ac96e951d1971fe0b4a3bbdf',
};
async function getGenerator(generator) {
    const generators = await config_1.getConfigKey('generators', {});
    if (!generators[generator]) {
        throw new Error(`Generator ${generator} is not defined`);
    }
    return generators[generator];
}
async function getGists() {
    return Promise.resolve(defaultGists);
}
async function cacheGist(id, cacheFile) {
    const { body } = await got(`https://api.github.com/gists/${id}`);
    const files = lodash_1.get(JSON.parse(body), 'files', {});
    const cache = Object.keys(files).reduce((acc, curr) => (Object.assign({}, acc, { [curr]: files[curr].content })), {});
    fs_1.writeFileSync(cacheFile, JSON.stringify({ files: cache }, null, 2));
}
async function cacheGists(cmd, cacheDir, gists) {
    fs_extra_1.ensureDirSync(cacheDir);
    const ps = Object.keys(gists).map(async (id) => {
        const cacheFile = path_1.join(cacheDir, gists[id]);
        if (!fs_1.existsSync(cacheFile)) {
            cmd.log(`Caching gist ${id}...`);
            return await cacheGist(gists[id], cacheFile);
        }
    });
    return Promise.all(ps);
}
const actions = {
    gist: {
        handler: async ({ cmd, args, flags }) => {
            const gists = await getGists();
            if (!Object.keys(gists).length) {
                return cmd.log(`No gists defined`);
            }
            const { generator } = args;
            const { output, force } = flags;
            if (!gists[generator]) {
                gists[generator] = generator;
            }
            const gistCache = path_1.join(cmd.config.configDir, 'gist-cache');
            await cacheGists(cmd, gistCache, gists);
            if (generator) {
                await generator_1.runGenerators(cmd, generator, output, force, name => {
                    const cacheFile = path_1.join(gistCache, gists[name]);
                    const gist = fs_1.readFileSync(cacheFile, 'utf-8');
                    return JSON.parse(gist);
                });
            }
            else {
                Object.keys(gists).forEach(async (id) => {
                    const cacheFile = path_1.join(gistCache, gists[id]);
                    const gist = fs_1.readFileSync(cacheFile, 'utf-8');
                    const files = lodash_1.get(JSON.parse(gist), 'files', {});
                    cli_ux_1.cli.table(Object.keys(files).map((name) => ({ name })), { name: { header: `Gist [${id}] (https://gist.github.com/${gists[id]}) files: ` } }, { printLine: cmd.log });
                });
            }
            return Promise.resolve();
        },
    },
    list: {
        handler: async ({ cmd }) => {
            const generators = await config_1.getConfigKey('generators', []);
            if (!Object.keys(generators).length) {
                return cmd.log(`No generators defined`);
            }
            Object.keys(generators).forEach(generator => {
                cli_ux_1.cli.table(Object.keys(generators[generator]).map((name) => ({ name })), { name: { header: `Generator [${generator}] files: ` } }, { printLine: cmd.log });
            });
            return Promise.resolve();
        },
    },
    run: {
        handler: async ({ args, cmd, flags }) => {
            const generators = await config_1.getConfigKey('generators', []);
            if (!Object.keys(generators).length) {
                return cmd.log(`No generators defined`);
            }
            const { generator } = args;
            const { output, force } = flags;
            if (!generator) {
                throw new Error(`Please specify which generator to run`);
            }
            await generator_1.runGenerators(cmd, generator, output, force, name => getGenerator(name));
            return Promise.resolve();
        },
    },
};
class Generate extends command_1.Command {
    async run() {
        // Get the args, flags from the Command
        const { args, flags } = this.parse(Generate);
        // Get a reference to the handler
        const handler = actions[args.action].handler;
        // We run the handler, pass in a reference to the object, the args and the flags.
        await handler({ cmd: this, args, flags });
    }
}
Generate.aliases = ['g'];
Generate.description = 'Run generators defined in the StackBlitz config and from GitHub Gists';
Generate.flags = {
    output: command_1.flags.string({ char: 'o', description: 'location to the output dir', required: false }),
    force: command_1.flags.boolean({
        char: 'f',
        description: 'force creation, overwriting existing files',
        required: false,
    }),
    help: command_1.flags.help({ char: 'h' }),
};
Generate.args = [
    {
        name: 'action',
        description: 'the action',
        required: true,
        options: Object.keys(actions),
    },
    {
        name: 'generator',
        description: 'the generator you want run',
    },
];
exports.default = Generate;
