"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer = require("inquirer");
const lodash_1 = require("lodash");
const config_1 = require("../config");
const utils_1 = require("../utils");
const actions = ['build', 'list', 'pull', 'push', 'run'];
class Image extends command_1.Command {
    async run() {
        const defaults = await config_1.getConfigKey('defaults.image', {});
        const images = await config_1.getConfigKey('images');
        if (!images) {
            throw Error(`No 'images' found in StackBlitz config.`);
        }
        const imageNames = Object.keys(images);
        const { args, flags } = this.parse(Image);
        const workspace = lodash_1.get(flags, 'workspace') || process.cwd();
        let { action, image } = args;
        if (!action) {
            let responses = await inquirer.prompt([
                {
                    name: 'action',
                    message: 'Select an action',
                    type: 'list',
                    choices: actions.map(name => ({ name })),
                },
            ]);
            action = responses.action;
        }
        if (!image && action !== 'list') {
            let responses = await inquirer.prompt([
                {
                    name: 'image',
                    message: 'Select an image',
                    type: 'list',
                    choices: imageNames.map(name => ({ name })),
                },
            ]);
            image = responses.image;
        }
        if (image && !images[image]) {
            throw new Error(`Could not find image ${image}. Pick one from: ${imageNames.join(', ')} `);
        }
        await this.handleAction(action, images[image], workspace, defaults);
    }
    async handleAction(action, image = {}, workspace, defaults) {
        let { args, name, cmd, path } = image;
        let { tag } = image;
        tag = tag || 'latest';
        let params = [...lodash_1.get(defaults, action, []), ...lodash_1.get(args, action, [])];
        let runCommand = true;
        switch (action) {
            case 'build': {
                params = [...params, '--cache-from', `${name}:${tag}`, '-t', `${name}:${tag}`, path ? path : workspace];
                cmd = '';
                break;
            }
            case 'pull': {
                params = [...params, `${name}:${tag}`];
                break;
            }
            case 'push': {
                params = [...params, `${name}:${tag}`];
                break;
            }
            case 'run': {
                params = [...params, `${name}:${tag}`];
                break;
            }
            case 'list': {
                this.log(JSON.stringify(await config_1.getConfigKey('images'), null, 2));
                runCommand = false;
                break;
            }
            default: {
                this.log(`Unhandled action: ${action}`);
            }
        }
        if (runCommand) {
            const command = utils_1.dockerCommand(action, [...params, cmd]);
            this.log(`🚀  Running command ${command}`);
            await utils_1.exec(command, workspace);
        }
    }
}
Image.description = 'Run operations on the images defined in StackBlitz config';
Image.flags = {
    help: command_1.flags.help({ char: 'h' }),
    workspace: command_1.flags.string({
        char: 'w',
        description: 'The project working directory',
    }),
};
Image.args = [
    {
        name: 'action',
        options: actions,
        description: 'Operation to run',
    },
    {
        name: 'image',
        description: 'Image to run operation on',
    },
];
exports.default = Image;
