"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
const lodash_1 = require("lodash");
const path_1 = require("path");
const utils_1 = require("../utils");
const depsOk = require('deps-ok');
function getCommandType(ext) {
    switch (ext) {
        case '.git':
            return 'git';
        case '.tgz':
            return 'tar';
    }
}
class Run extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(Run);
        let { workspace } = args;
        let { check, cmd, cwd, skipInstall, skipStart, template } = flags;
        let command;
        check = check || 'package.json';
        cwd = cwd ? path_1.join(workspace, cwd) : workspace;
        if (template) {
            const commandType = getCommandType(path_1.extname(template));
            switch (commandType) {
                case 'git':
                    if (!template.startsWith('https://github.com')) {
                        template = `https://github.com/${template.replace('.git', '')}/tarball/master`;
                    }
                    command = `wget -q -O - ${template} | tar xz --strip-components=1 -C ${workspace}`;
                    break;
                case 'tar':
                    command = `tar xfz ${template} --strip-components=2 -C ${workspace}`;
                    break;
            }
            if (!command) {
                throw new Error(`Don't know how to provision type ${commandType} in ${template}`);
            }
            const templateExists = await fs_1.existsSync(template);
            if (commandType === 'tar' && !templateExists) {
                throw new Error(`Template does not exist: ${template}`);
            }
        }
        if (!(await utils_1.workspaceExists(workspace))) {
            try {
                await fs_extra_1.ensureDir(workspace);
                this.log(`💥  Created workspace ${workspace}`);
            }
            catch (e) {
                this.error(e);
                throw new Error(e);
            }
        }
        const hasProject = utils_1.projectCheck(workspace, check);
        if (check && command && template && !hasProject) {
            this.log(`🚀  Creating new project in ${workspace} from ${template}`);
            await utils_1.exec(command, workspace);
        }
        else {
            this.log(`✅  Found project in ${workspace}`);
        }
        if (!cwd) {
            throw new Error(`Current working dir '${cwd}' is not defined, use -w to set it`);
        }
        else {
            this.debug(`DEBUG: Using cwd: ${cwd}`);
        }
        const canInstall = utils_1.workspaceCanInstall(cwd);
        if (!skipInstall && canInstall) {
            if (!depsOk(cwd, { verbose: false })) {
                this.log('🎁  Installing dependencies');
                await utils_1.exec('yarn --ignore-engines', cwd);
            }
            else {
                this.log('✅  Dependencies are up to date');
            }
        }
        else {
            this.log('SKIPPED: Installing dependencies');
        }
        const scripts = lodash_1.get(utils_1.projectPackage(cwd), 'scripts');
        // Determine which start script to use
        const startScripts = ['sb-start', 'dev', 'start:dev', 'start'];
        startScripts.forEach(script => {
            if (!cmd && scripts[script]) {
                cmd = `yarn ${script}`;
            }
        });
        if (!cmd) {
            cmd = 'yarn start';
        }
        if (!skipStart && canInstall) {
            this.log('🔥  Starting project');
            await utils_1.exec(cmd, cwd);
        }
        else {
            this.log('SKIPPED: Starting Project');
        }
    }
}
Run.description = 'Run project in a workspace, optionally provisioning it with a template or GitHub project.';
Run.args = [
    {
        name: 'workspace',
        default: process.cwd(),
        required: true,
        description: 'The project working directory',
    },
];
Run.flags = {
    help: command_1.flags.help({
        char: 'h',
    }),
    cmd: command_1.flags.string({
        char: 'c',
        description: 'Command to execute',
    }),
    check: command_1.flags.string({
        char: 'k',
        default: './package.json',
        description: 'Location to file to check if project exists or needs provisioning',
    }),
    skipInstall: command_1.flags.boolean({
        char: 'i',
        default: false,
        description: 'Install dependencies',
    }),
    skipStart: command_1.flags.boolean({
        char: 's',
        default: false,
        description: 'Only provision the project but do not start it',
    }),
    template: command_1.flags.string({
        char: 't',
        description: 'The file or url used to provision the workspace',
    }),
    cwd: command_1.flags.string({
        char: 'w',
        description: 'The current working directory where commands are executed',
    }),
};
exports.default = Run;
