"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const config_1 = require("../config");
const utils_1 = require("../utils");
const cli_ux_1 = require("cli-ux");
const kleur_1 = require("kleur");
const label = (text) => `[${kleur_1.white(kleur_1.bold(text))}]`;
async function getTask(task) {
    const tasks = await config_1.getConfigKey('tasks', []);
    if (!tasks[task]) {
        throw new Error(`Task ${task} is not defined`);
    }
    return tasks[task];
}
const actions = {
    list: {
        handler: async ({ cmd }) => {
            const tasks = await config_1.getConfigKey('tasks', []);
            if (!Object.keys(tasks).length) {
                return cmd.log(`No tasks defined`);
            }
            Object.keys(tasks).forEach(task => {
                cli_ux_1.cli.table(tasks[task].map((name) => ({ name })), { name: { header: `Task [${task}] steps: ` } }, { printLine: cmd.log });
            });
            return Promise.resolve();
        },
    },
    run: {
        handler: async ({ cmd, args }) => {
            const tasks = await config_1.getConfigKey('tasks', []);
            if (!Object.keys(tasks).length) {
                return cmd.log(`No tasks defined`);
            }
            const { task } = args;
            if (!task) {
                throw new Error(`Please specify which task to run`);
            }
            // The task argument can have multiple comma-separated tasks
            const allTasks = task.includes(',') ? task.split(',') : [task];
            const totalTasks = allTasks.length;
            cmd.log(kleur_1.white(kleur_1.bold(`Running ${totalTasks} task(s): ${allTasks.join(', ')}`)));
            // Look over each of the tasks we need to run
            for (const [taskIndex, currentTask] of allTasks.entries()) {
                // Get the steps of the task
                const steps = await getTask(currentTask);
                // Loop over each of the steps in this task
                for (const [stepIndex, step] of steps.entries()) {
                    const task = label(currentTask);
                    const stepCount = label(stepIndex + 1 + '/' + steps.length);
                    const taskCount = label(taskIndex + 1 + '/' + totalTasks);
                    const message = `Task ${taskCount} ${task} step ${stepCount}: ${kleur_1.gray(step)}`;
                    cmd.log(message);
                    await utils_1.exec(step);
                }
            }
            cmd.log(kleur_1.green(kleur_1.bold(`Done running ${totalTasks} task(s): ${allTasks.join(', ')}`)));
            return Promise.resolve();
        },
    },
};
class Task extends command_1.Command {
    async run() {
        // Get the args, flags from the Command
        const { args, flags } = this.parse(Task);
        // Get a reference to the handler
        const handler = actions[args.action].handler;
        // We run the handler, pass in a reference to the object, the args and the flags.
        await handler({ cmd: this, args, flags });
    }
}
Task.description = 'Run tasks defined in the StackBlitz config';
Task.flags = { help: command_1.flags.help({ char: 'h' }) };
Task.args = [
    {
        name: 'action',
        description: 'the action',
        required: true,
        options: Object.keys(actions),
    },
    {
        name: 'task',
        description: 'the task you want run',
    },
];
exports.default = Task;
