"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const fs_extra_1 = require("fs-extra");
const kleur_1 = require("kleur");
const path_1 = require("path");
const inquirer = require("inquirer");
exports.label = (text) => `[${kleur_1.white(kleur_1.bold(text))}]`;
const replaceVars = (str = '', values) => str.replace(/{(.*?)}/g, (match, offset) => values[offset]);
async function runGenerators(cmd, generator, output, force = false, resolver) {
    // The generator argument can have multiple comma-separated generators
    const allGenerators = generator.includes(',') ? generator.split(',') : [generator];
    const totalGenerators = allGenerators.length;
    cmd.log(kleur_1.white(kleur_1.bold(`Running ${totalGenerators} generator(s): ${allGenerators.join(', ')}. ${output ? `Output dir: ${output}` : ''}`)));
    output = output || process.cwd();
    return Promise.resolve()
        .then(async () => {
        // Look for variables in each of the generators
        for (const [_, currentGenerator] of allGenerators.entries()) {
            const generator = await resolver(currentGenerator);
            const vars = generator.vars || {};
            const questions = Object.keys(vars).map(varName => {
                const v = vars[varName];
                return {
                    name: varName,
                    message: v.description,
                    default: v.default.charAt(0) === '$' ? process.env[v.default.substring(1)] : v.default,
                };
            });
            return inquirer.prompt(questions);
        }
    })
        .then(async (responses) => {
        // Loop over each of the generators we need to run
        for (const [generatorIndex, currentGenerator] of allGenerators.entries()) {
            // Get the steps of the generator
            const generator = await resolver(currentGenerator);
            // Loop over each of the files in this generator
            Object.keys(generator.files).forEach(async (filename, stepIndex) => {
                const generatorName = exports.label(currentGenerator);
                const fileCount = exports.label(stepIndex + 1 + '/' + Object.keys(generator.files).length);
                const generatorCount = exports.label(generatorIndex + 1 + '/' + totalGenerators);
                const message = `Generator ${generatorCount} ${generatorName} file ${fileCount}: ${kleur_1.gray(filename)}`;
                cmd.log(message);
                const path = path_1.join(output, filename);
                if (fs_1.existsSync(path) && force === false) {
                    cmd.log(kleur_1.yellow(`File ${path} already exists, skipping... Use -f to overwrite existing files.`));
                }
                else {
                    const filePath = replaceVars(path, responses);
                    const contents = replaceVars(generator.files[filename].toString(), responses);
                    fs_extra_1.ensureDirSync(path_1.dirname(filePath));
                    fs_1.writeFileSync(filePath, contents);
                }
            });
        }
        return Promise.resolve();
    })
        .then(() => {
        cmd.log(kleur_1.green(kleur_1.bold(`Done running ${totalGenerators} generator(s): ${allGenerators.join(', ')}`)));
    });
}
exports.runGenerators = runGenerators;
